{# def
type="submit",
id="",
name="action",
value="submit",
onclick="",
target="",
swap="",
get="",
select="",
after_request="",
vals="",
full_width=false,
hidden=false,
aria_label=""
#}
{% set css = "text-white bg-primary-600 hover:bg-primary-700 focus:ring-4 focus:outline-none focus:ring-primary-300
font-medium rounded-lg text-sm px-5 py-2.5 text-center dark:bg-primary-600 dark:hover:bg-primary-700
dark:focus:ring-primary-800" %}
<button type="{{type}}" {%if id %}id="{{id}}" {%endif%} name="{{name}}" value="{{value}}" {% if
  target%}hx-target="{{target}}" {% endif %} {% if swap %}hx-swap="{{swap}}" {% endif %} {% if select
  %}hx-select="{{select}}" {% endif %} {% if get %}hx-get="{{get}}" {% endif %} {% if onclick %}onclick="{{onclick}}" {%
  endif %}{% if after_request %}hx-on::after-request="{{after_request}}" {% endif %} {% if vals
  %}hx-vals='{{vals|safe}}' {% endif %} {% if aria_label %}aria-label="{{aria_label}}" {% endif %}
  class="{% if full_width %}w-full {% endif %}{{css}}" {% if hidden %}hidden{% endif %}>
  {{ content}}
</button>
