{# def name, value="", input_type="text", id="", aria_label="", placeholder="" #}
{% set css = "bg-neutral-50 border border-neutral-300 text-neutral-900 text-base rounded-lg focus:ring-primary-500
focus:border-primary-500 block w-full p-2.5 dark:bg-neutral-700 dark:border-neutral-600 dark:placeholder-neutral-400
dark:text-white dark:focus:ring-primary-500 dark:focus:border-primary-500" %}

<input name="{{name}}" value="{{value}}" type="{{input_type}}" id="{{id}}" {% if aria_label
  %}aria-label="{{aria_label}}" {% endif %} {% if placeholder %}placeholder="{{placeholder}}" {% endif %}
  class="{{css}}" />
