{# def widget #}
<pydantic_form.Widget widget={widget} removable={widget.removable}>
  <div class="pt-4">
    <Label for={widget.id}>{{widget.title}}</Label>
    <Checkbox name={widget.name} value={widget.value} type="checkbox" id={widget.id} />
  </div>
</pydantic_form.Widget>
